/************************************************************************/
/* Often when a web based query is performed, there is a lot of search- */
/* ing, collating, and joining to be done.  This is a tool to help man- */
/* age a cache of information stored in a temp-table.  We store the     */
/* information in a disk file and between hits we can again reference   */
/* that disk file between multiple agents or what have you.             */
/* TODO: A lot of this is handled better in 10.1B with XML, but we      */
/* don't have that option in 10.1A or earlier.                          */
/*                                                                      */
/* Written By Scott Auge                                                */
/* sauge@amduus.com scott_auge@yahoo.com                                */
/************************************************************************/

{MakeID3.i}

/************************************************************************/
/* Once we have read in the cache contents, we will need this for later */
/* procedure calls.  So keep it available to an instance of this proc-  */
/* cedure.                                                              */
/************************************************************************/

DEFINE VARIABLE ttGlobalHdl AS HANDLE NO-UNDO.   
DEFINE VARIABLE ErrorMessage AS CHARACTER INIT "NONE" NO-UNDO.

CREATE TEMP-TABLE ttGlobalHdl.

/************************************************************************/
/* Given a temp-table handle, write it's contents to disk.  Given the   */
/* fields in the same order, its should naturally populate the fields   */
/* return loads.                                                        */
/************************************************************************/
        
PROCEDURE WriteDataToDisk:

  DEFINE INPUT PARAMETER ttHdl AS HANDLE NO-UNDO.
  DEFINE OUTPUT PARAMETER cFileName AS CHARACTER NO-UNDO.  
  
  DEFINE VARIABLE QueryHdl AS HANDLE NO-UNDO.
  DEFINE VARIABLE CurrentFieldNumber AS INTEGER NO-UNDO.
  DEFINE VARIABLE FieldHdl AS HANDLE NO-UNDO.
  DEFINE VARIABLE Query4GL AS CHARACTER NO-UNDO.
  
  ASSIGN cFileName = "/tmp/cachemgr_p_" + MakeID3(10).
  ASSIGN Query4GL = "FOR EACH " + ttHdl:Name + " NO-LOCK".
  
  CREATE QUERY QueryHdl.
  
  /* ttHdl:BUFFER-CREATE(). */
  
  QueryHdl:ADD-BUFFER(ttHdl:DEFAULT-BUFFER-HANDLE).   
  
  IF QueryHdl:QUERY-PREPARE (Query4GL) THEN DO:
      
    QueryHdl:QUERY-OPEN.
    
    OUTPUT TO VALUE (cFileName).
    
    REPEAT:
    
      QueryHdl:GET-NEXT().
      IF QueryHdl:QUERY-OFF-END THEN LEAVE.
        
      DO CurrentFieldNumber = 1 TO ttHdl:DEFAULT-BUFFER-HANDLE:NUM-FIELDS:
        
        FieldHdl = ttHdl:DEFAULT-BUFFER-HANDLE:BUFFER-FIELD(CurrentFieldNumber).
        EXPORT STRING (FieldHdl:BUFFER-VALUE).
        
      END. /* DO */
        
    END. /* DO */                                                                                                                     
    
    OUTPUT CLOSE.
    
    QueryHdl:QUERY-CLOSE(). 
    
    ASSIGN ErrorMessage = "NONE".
                     
  END.        
  ELSE
    ASSIGN ErrorMessage = ERROR-STATUS:GET-MESSAGE(ERROR-STATUS:NUM-MESSAGES).
  
  DELETE OBJECT QueryHdl.

END. /* PROCEDURE WriteDataToDisk */

/************************************************************************/
/* When we read data from disk, we actually store it in a temp-table    */
/* in the instance of the procedure.  Other internal procedures will    */
/* need to make use of it.                                              */
/************************************************************************/

PROCEDURE ReadDataFromDisk:

  DEFINE INPUT PARAMETER cFileName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER TableHandle AS HANDLE NO-UNDO.
  
  DEFINE VARIABLE iCurrentField AS INTEGER NO-UNDO.
  DEFINE VARIABLE cLine AS CHARACTER NO-UNDO.
  DEFINE VARIABLE FieldHdl AS HANDLE NO-UNDO.
  
  /****************************************************************/
  /* This will get our schema in local temp-table                 */
  /****************************************************************/
  
  ttGlobalHdl:COPY-TEMP-TABLE(TableHandle).

  /****************************************************************/  
  /* Start populating our local temp-table from the cache.        */
  /****************************************************************/
  
  INPUT FROM VALUE (cFileName).
  
  REPEAT ON END-KEY UNDO, LEAVE:
  
    ttGlobalHdl:DEFAULT-BUFFER-HANDLE:BUFFER-CREATE().
  
    DO iCurrentField = 1 TO ttGlobalHdl:DEFAULT-BUFFER-HANDLE:NUM-FIELDS:
    
      IMPORT cLine.
      FieldHdl = ttGlobalHdl:DEFAULT-BUFFER-HANDLE:BUFFER-FIELD(iCurrentField).
      
      /* PUT UNFORMATTED FIELDHDL:DATA-TYPE " " cLine "<BR>". */
      
      IF FieldHdl:DATA-TYPE = "INTEGER" THEN
        FieldHdl:BUFFER-VALUE = INTEGER(cLine).
      ELSE IF FieldHdl:DATA-TYPE = "date" THEN 
        FieldHdl:BUFFER-VALUE = DATE(cLine).
      ELSE IF FieldHdl:DATA-TYPE BEGINS "decimal" THEN
        FieldHdl:BUFFER-VALUE = DECIMAL(cLine).
      ELSE
        FieldHdl:BUFFER-VALUE = cLine.
      
    END. /* DO */
  
  END. /* REPEAT */
  
  INPUT CLOSE.
  
  /****************************************************************/
  /* Our REPEAT seems to leave an empty buffer hanging around.    */
  /****************************************************************/
  
  ttGlobalHdl:DEFAULT-BUFFER-HANDLE:BUFFER-DELETE().
  
  
END. /* PROCEDURE ReadDataFromDisk */

/************************************************************************/
/* Delete the cache.   We don't bother deleting the temp-table data     */
/* stored in the instance because we can clear that simply by making a  */
/* new instance!                                                        */
/************************************************************************/

PROCEDURE ClearData:

  DEFINE INPUT PARAMETER cFileName AS CHARACTER NO-UNDO.
  
  OS-DELETE VALUE (cFileName). 
  
END. /* PROCEDURE ClearData */

/************************************************************************/
/* After loading the cache, ask for how many pages are available.       */
/************************************************************************/

PROCEDURE DetermineNumberPages:

  DEFINE INPUT PARAMETER RowsPerPage AS INTEGER NO-UNDO.
  DEFINE OUTPUT PARAMETER NumberOfPages AS INTEGER NO-UNDO.
  
  DEFINE VARIABLE Query4GL AS CHARACTER NO-UNDO.
  DEFINE VARIABLE QueryHdl AS HANDLE NO-UNDO.
  DEFINE VARIABLE iCount AS INTEGER NO-UNDO.
  DEFINE VARIABLE FractionalNumberOfPages AS DECIMAL NO-UNDO.
  
  ASSIGN Query4GL = "FOR EACH " + ttGlobalHdl:Name + " NO-LOCK".
  
  CREATE QUERY QueryHdl.
  
  QueryHdl:ADD-BUFFER(ttGlobalHdl:DEFAULT-BUFFER-HANDLE).
  
  ASSIGN iCount = 0.
  
  IF QueryHdl:QUERY-PREPARE (Query4GL) THEN DO:
    
    QueryHdl:QUERY-OPEN.
    
    REPEAT:
    
      QueryHdl:GET-NEXT().
      IF QueryHdl:QUERY-OFF-END THEN LEAVE.
        
      ASSIGN iCount = iCount + 1.
        
    END. /* DO */
    
    
    QueryHdl:QUERY-CLOSE().
    
  END.      
  ELSE
    ASSIGN ErrorMessage = ERROR-STATUS:GET-MESSAGE(ERROR-STATUS:NUM-MESSAGES).  
  
  DELETE OBJECT QueryHdl.

  ASSIGN FractionalNumberOfPages = iCount / RowsPerPage.  
  
  /* PUT UNFORMATTED iCount " " FractionalNumberOfPages "<br>". */
  
  IF FractionalNumberOfPages - INTEGER (FractionalNumberOfPages) <> 0.0 THEN
    ASSIGN NumberOfPages = INTEGER (FractionalNumberOfPages) + 1.
  ELSE
    ASSIGN NumberOfPages = INTEGER (FractionalNumberOfPages).
  
END. /* PROCEDURE DetermineNumberPages */

/************************************************************************/
/* Send in the handle of another temp-table LIKE the original data      */
/* temp table to return only that page of data from the cache.          */
/************************************************************************/

PROCEDURE ReturnPageResults:

  DEFINE INPUT PARAMETER PageNumberDesired AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER RowsPerPage AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER ttTableHandle AS HANDLE.
  
  DEFINE VARIABLE iCurrentRowCount AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCurrentPageNumber AS INTEGER NO-UNDO.
  DEFINE VARIABLE Query4GL AS CHARACTER NO-UNDO.
  DEFINE VARIABLE QueryHdl AS HANDLE NO-UNDO.
  
  ASSIGN Query4GL = "FOR EACH " + ttGlobalHdl:Name + " NO-LOCK".
  
  CREATE QUERY QueryHdl.
  
  QueryHdl:ADD-BUFFER(ttGlobalHdl:DEFAULT-BUFFER-HANDLE).
  
  ASSIGN iCurrentRowCount = 0
         iCurrentPageNumber = 0.
  
  IF QueryHdl:QUERY-PREPARE (Query4GL) THEN DO:
    
    QueryHdl:QUERY-OPEN.
    
    REPEAT:
    
      QueryHdl:GET-NEXT().
      IF QueryHdl:QUERY-OFF-END THEN LEAVE.
        
      IF iCurrentRowCount MOD RowsPerPage = 0 THEN ASSIGN iCurrentPageNumber = iCurrentPageNumber + 1.        
      ASSIGN iCurrentRowCount = iCurrentRowCount + 1.
        
      IF iCurrentPageNumber = PageNumberDesired THEN DO:
      
        ttTableHandle:DEFAULT-BUFFER-HANDLE:BUFFER-CREATE().        
        ttTableHandle:DEFAULT-BUFFER-HANDLE:BUFFER-COPY(ttGlobalHdl:DEFAULT-BUFFER-HANDLE).
      
      END. /* IF */
      
    END. /* REPEAT */
    
    QueryHdl:QUERY-CLOSE().
    
  END.   /* IF QUERY-PREPARE */   
  ELSE
    ASSIGN ErrorMessage = ERROR-STATUS:GET-MESSAGE(ERROR-STATUS:NUM-MESSAGES).  
  
  DELETE OBJECT QueryHdl.  
  
  
END. /* PROCEDURE REturnPageResults */

/************************************************************************/  
/* Allow the calling program to retrieve an error message that may have */
/* occurred.                                                            */
/************************************************************************/  

PROCEDURE GetError:
  
  DEFINE OUTPUT PARAMETER cMsg AS CHARACTER NO-UNDO.

  ASSIGN cMsg = ErrorMessage.
  
END.

